/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* MPIA, NEVEC - MIDI project
*
* "@(#) $Id: handle_exceptions.c,v 1.4 2002/05/08 21:34:35 jdejong Exp $"
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jdejong    2002-05-08 imported
*
*/
/************************************************************************
 $Id: handle_exceptions.c,v 1.4 2002/05/08 21:34:35 jdejong Exp $
 ------------------------------------------------------------------------
Exception handling routines for FITS files. These are needed to handle
format problems which occur deep in the FITS file routines without having
to stop the program at an inappropiate time. E.g. when a certain FITS
table does not occur in the file which has been read. 
************************************************************************/

#include "vltPort.h"
#include "handle_exceptions.h"
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

/* array of exceptions */
char *exception_list[MAX_NUM_EXCEPTIONS];

/* current index for the exceptions list */
int curr_excep_idx=-1;

/* throw an exception with a message, which is formatted as in printf 
	 Inputs:
	 fmt: format and arguments as in printf
*/
void throw_exception(char* fmt, ...) 
{
	/* scan variable argument list */
	va_list args;
	va_start (args,fmt);

  /* initialize exception pointers */
  if (curr_excep_idx==-1)
  {
    int i;
    for (i=0; i<MAX_NUM_EXCEPTIONS; i++)
      exception_list[i] = NULL;
    curr_excep_idx=0;
  }

	/* reset list if maximum number of exceptions has been reached */
	if (curr_excep_idx==MAX_NUM_EXCEPTIONS) 
		clear_exceptions();

	/* allocate memory for exception */
	if (exception_list[curr_excep_idx]!=NULL) 
		free(exception_list[curr_excep_idx]);
	exception_list[curr_excep_idx] = 
		(char*)malloc(sizeof(char)*MAX_EXCEPTION_SIZE);

	/* print message body */
	vsprintf (exception_list[curr_excep_idx],fmt,args);

	/* next exception */
	curr_excep_idx++;

}

/* clear the exception stack */
void clear_exceptions() 
{
	int i;
	for (i=0; i<curr_excep_idx; i++) 
		free(exception_list[i]);
	curr_excep_idx=0;
}

/*  
Check whether the exceptions start with a certain string 
input: 
str - String to check for
returns:
-1 if no exception starts with the string, otherwise
an index of the exception containing the string is returned
*/
int check_exceptions(char* str) 
{
	int i;
	for (i=0; i<curr_excep_idx; i++) {
		if (!strncmp(exception_list[i],str,strlen(str)))
			return(i);
	}
	return(-1); /* str was not found */
}

/*
Get the number of the current exception
This is necessary because the global variable cannot be accessed by
means of a header file
*/
int get_exception_idx() {
	return(curr_excep_idx);
}

/* Show exception number i */
char* get_exception(int idx) {
	return (exception_list[idx]);
}






